from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class BirthdayMigration(MigrationABC):
    name = "1.2.0_BirthdayMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE Users
                    ADD Birthday DATE NULL AFTER MessageCount;
                """
            )
        )

        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE UsersHistory
                    ADD Birthday DATE NULL AFTER MessageCount;
                """
            )
        )
        self._exec(__file__, "users.sql")

        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_Server
                    ADD XpForBirthday BIGINT(20) NOT NULL DEFAULT 0 AFTER XpPerAchievement;
                """
            )
        )

        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_ServerHistory
                    ADD XpForBirthday BIGINT(20) NOT NULL DEFAULT 0 AFTER XpPerAchievement;
                """
            )
        )
        self._exec(__file__, "config/server.sql")

    def downgrade(self):
        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE Users DROP COLUMN Birthday;
                """
            )
        )
        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE UsersHistory DROP COLUMN Birthday;
                """
            )
        )
        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_Server DROP COLUMN XpForBirthday;
                """
            )
        )
        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_ServerHistory DROP COLUMN XpForBirthday;
                """
            )
        )
