from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.steam_special_offer_repository_abc import SteamSpecialOfferRepositoryABC
from bot_data.model.steam_special_offer import SteamSpecialOffer


class SteamSpecialOfferRepositoryService(SteamSpecialOfferRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        servers: ServerRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._servers = servers

        SteamSpecialOfferRepositoryABC.__init__(self)

    @staticmethod
    def _get_value_from_result(value: any) -> Optional[any]:
        if isinstance(value, str) and "NULL" in value:
            return None

        return value

    def _steam_special_offer_from_result(self, sql_result: tuple) -> SteamSpecialOffer:
        return SteamSpecialOffer(
            self._get_value_from_result(sql_result[1]),  # name
            float(self._get_value_from_result(sql_result[2])),  # original_price
            float(self._get_value_from_result(sql_result[3])),  # discount_price
            int(self._get_value_from_result(sql_result[4])),  # discount_pct
            id=self._get_value_from_result(sql_result[0]),  # id
        )

    def get_steam_special_offers(self) -> List[SteamSpecialOffer]:
        steam_special_offers = List(SteamSpecialOffer)
        self._logger.trace(__name__, f"Send SQL command: {SteamSpecialOffer.get_select_all_string()}")
        results = self._context.select(SteamSpecialOffer.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get steam_special_offer with id {result[0]}")
            steam_special_offers.append(self._steam_special_offer_from_result(result))

        return steam_special_offers

    def get_steam_special_offer_by_name(self, name: str) -> SteamSpecialOffer:
        self._logger.trace(__name__, f"Send SQL command: {SteamSpecialOffer.get_select_by_name_string(name)}")
        result = self._context.select(SteamSpecialOffer.get_select_by_name_string(name))[0]

        return self._steam_special_offer_from_result(result)

    def add_steam_special_offer(self, steam_special_offer: SteamSpecialOffer):
        self._logger.trace(__name__, f"Send SQL command: {steam_special_offer.insert_string}")
        self._context.cursor.execute(steam_special_offer.insert_string)

    def update_steam_special_offer(self, steam_special_offer: SteamSpecialOffer):
        self._logger.trace(__name__, f"Send SQL command: {steam_special_offer.udpate_string}")
        self._context.cursor.execute(steam_special_offer.udpate_string)

    def delete_steam_special_offer(self, steam_special_offer: SteamSpecialOffer):
        self._logger.trace(__name__, f"Send SQL command: {steam_special_offer.delete_string}")
        self._context.cursor.execute(steam_special_offer.delete_string)
