import json

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.model.technician_config import TechnicianConfig
from bot_data.model.technician_id_config import TechnicianIdConfig
from bot_data.model.technician_ping_url_config import TechnicianPingUrlConfig


class TechnicianConfigRepositoryService(TechnicianConfigRepositoryABC):
    def __init__(self, logger: DatabaseLogger, db_context: DatabaseContextABC):
        TechnicianConfigRepositoryABC.__init__(self)

        self._logger = logger
        self._context = db_context

    def _get_technician_ids(self) -> List[int]:
        ids = List(int)
        self._logger.trace(__name__, f"Send SQL command: {TechnicianIdConfig.get_select_all_string()}")
        results = self._context.select(TechnicianIdConfig.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Got TechnicianId with id {result[0]}")
            ids.append(result[1])

        return ids

    def _get_technician_ping_urls(self) -> List[str]:
        urls = List(str)
        self._logger.trace(__name__, f"Send SQL command: {TechnicianPingUrlConfig.get_select_all_string()}")
        results = self._context.select(TechnicianPingUrlConfig.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Got TechnicianPingUrl with id {result[0]}")
            urls.append(result[1])

        return urls

    def _from_result(self, result: tuple) -> TechnicianConfig:
        return TechnicianConfig(
            result[1],
            result[2],
            result[3],
            result[4],
            json.loads(result[5]),
            self._get_technician_ids(),
            self._get_technician_ping_urls(),
            result[6],
            result[7],
            id=result[0],
        )

    def does_technician_config_exists(self) -> bool:
        self._logger.trace(__name__, f"Send SQL command: {TechnicianConfig.get_select_all_string()}")
        result = self._context.select(TechnicianConfig.get_select_all_string())

        return len(result) > 0

    def get_technician_config(self) -> TechnicianConfig:
        self._logger.trace(__name__, f"Send SQL command: {TechnicianConfig.get_select_all_string()}")
        result = self._context.select(TechnicianConfig.get_select_all_string())[0]

        return self._from_result(result)

    def add_technician_config(self, technician_config: TechnicianConfig):
        self._logger.trace(__name__, f"Send SQL command: {technician_config.insert_string}")
        self._context.cursor.execute(technician_config.insert_string)

    def update_technician_config(self, technician_config: TechnicianConfig):
        self._logger.trace(__name__, f"Send SQL command: {technician_config.udpate_string}")
        self._context.cursor.execute(technician_config.udpate_string)

    def delete_technician_config(self, technician_config: TechnicianConfig):
        self._logger.trace(__name__, f"Send SQL command: {technician_config.delete_string}")
        self._context.cursor.execute(technician_config.delete_string)

    def add_technician_id_config(self, technician_id: TechnicianIdConfig):
        self._logger.trace(__name__, f"Send SQL command: {technician_id.insert_string}")
        self._context.cursor.execute(technician_id.insert_string)

    def update_technician_id_config(self, technician_id: TechnicianIdConfig):
        self._logger.trace(__name__, f"Send SQL command: {technician_id.udpate_string}")
        self._context.cursor.execute(technician_id.udpate_string)

    def delete_technician_id_config(self, technician_id: TechnicianIdConfig):
        self._logger.trace(__name__, f"Send SQL command: {technician_id.delete_string}")
        self._context.cursor.execute(technician_id.delete_string)

    def add_technician_ping_url_config(self, technician_ping_url: TechnicianPingUrlConfig):
        self._logger.trace(__name__, f"Send SQL command: {technician_ping_url.insert_string}")
        self._context.cursor.execute(technician_ping_url.insert_string)

    def update_technician_ping_url_config(self, technician_ping_url: TechnicianPingUrlConfig):
        self._logger.trace(__name__, f"Send SQL command: {technician_ping_url.udpate_string}")
        self._context.cursor.execute(technician_ping_url.udpate_string)

    def delete_technician_ping_url_config(self, technician_ping_url: TechnicianPingUrlConfig):
        self._logger.trace(__name__, f"Send SQL command: {technician_ping_url.delete_string}")
        self._context.cursor.execute(technician_ping_url.delete_string)
