import datetime

from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import tasks

from bot_core.abc.task_abc import TaskABC
from bot_core.logging.task_logger import TaskLogger
from bot_core.service.message_service import MessageService
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server_config import ServerConfig


class BirthdayWatcher(TaskABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: TaskLogger,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        users: UserRepositoryABC,
        message_service: MessageService,
        t: TranslatePipe,
    ):
        TaskABC.__init__(self)

        self._config = config
        self._logger = logger
        self._bot = bot
        self._db = db
        self._users = users
        self._message_service = message_service
        self._t = t

        self.watch.start()

    @tasks.loop(time=datetime.time(hour=8, minute=0))
    async def watch(self):
        self._logger.info(__name__, "Watching birthdays")
        try:
            today = datetime.date.today()
            users = self._users.get_users().where(lambda x: x.birthday is not None)
            for user in users:
                if user.birthday.day != today.day or user.birthday.month != today.month:
                    continue

                settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{user.server.discord_id}")

                user.xp += settings.xp_for_birthday
                self._users.update_user(user)
                self._db.save_changes()

                guild = self._bot.get_guild(user.server.discord_id)
                member = guild.get_member(user.discord_id)
                await self._message_service.send_channel_message(
                    self._bot.get_channel(settings.notification_chat_id),
                    self._t.transform("modules.base.user.birthday.has_birthday").format(member.mention),
                    is_persistent=True,
                )
        except Exception as e:
            self._logger.error(__name__, f"Watching birthdays failed", e)

    @watch.before_loop
    async def wait(self):
        await self._wait_until_ready()
