export class Queries {

  static guildsQuery = `
    query GuildsQuery($id: ID, $filter: ChannelFilter) {
      discord {
        guilds(filter: {id: $id}) {
          id
          name

          channels(filter: $filter) {
            id
            name
            type
          }
          roles {
            id
            name
          }
          emojis {
            id
            name
            url
          }
        }
      }
    }
  `;

  static discordUsersQuery = `
    query DiscordUsersQuery {
      discord {
        users {
          id
          name
        }
      }
    }
  `;

  static serverConfigDiscordQuery = `
    query ServerConfigDiscordQuery($id: ID) {
      discord {
        guilds(filter: {id: $id}) {
          id
          name

          roles {
            id
            name
          }

          channels {
            id
            name
            type
          }
        }
      }
    }
  `;

  static serversQuery = `
    query ServerInfo($filter: ServerFilter, $page: Page, $sort: Sort) {
      serverCount
      servers(filter: $filter, page: $page, sort: $sort) {
        id
        discordId
        name
        iconURL
        userCount
        clients {
          id
          discordId
          name
          sentMessageCount
          receivedMessageCount
          deletedMessageCount
          receivedCommandCount
          movedUsersCount
        }
      }
    }
  `;

  static hasServerFeatureFlag = `
    query HasServerFeatureFlag($filter: ServerFilter, $flag: String) {
      servers(filter: $filter) {
        hasFeatureFlag(flag: $flag) {
          key
          value
        }
      }
    }
  `;

  static gameServerQuery = `
      query GameServersList($serverId: ID) {
      servers(filter: {id: $serverId}) {
        gameServers {
          name
        }
      }
    }
  `;

  static levelQuery = `
    query LevelsList($serverId: ID, $filter: LevelFilter, $page: Page, $sort: Sort) {
      servers(filter: {id: $serverId}) {
        levelCount
        levels(filter: $filter, page: $page, sort: $sort) {
          id
          name
          color
          minXp
          permissions
          server {
            id
            name
          }
          createdAt
          modifiedAt
        }
      }
    }
  `;

  static levelWithHistoryQuery = `
    query LevelHistory($serverId: ID, $id: ID) {
      servers(filter: {id: $serverId}) {
        levelCount
        levels(filter: {id: $id}) {
          id

          history {
            id
            name
            color
            minXp
            permissions
            server
            deleted
            dateFrom
            dateTo
          }
        }
      }
    }
  `;

  static achievementTypeQuery = `
    query AchievementType {
      achievementOperators
      achievementAttributes {
        name
        type
      }
    }
  `;

  static achievementQuery = `
    query AchievementList($serverId: ID, $filter: AchievementFilter, $page: Page, $sort: Sort) {
      servers(filter: {id: $serverId}) {
        achievementCount
        achievements(filter: $filter, page: $page, sort: $sort) {
          id
          name
          description
          attribute
          operator
          value
          server {
            id
            name
          }
          createdAt
          modifiedAt
        }
      }
    }
  `;

  static achievementWithHistoryQuery = `
    query AchievementHistory($serverId: ID, $id: ID) {
      servers(filter: {id: $serverId}) {
        achievementCount
        achievements(filter: {id: $id}) {
          id

          history {
            id
            name
            description
            attribute
            operator
            value
            server
            deleted
            dateFrom
            dateTo
          }
        }
      }
    }
  `;


  static shortRoleNamePositionsQuery = `
    query {
      shortRoleNamePositions
    }
  `;

  static shortRoleNameQuery = `
    query ShortRoleNameList($serverId: ID, $filter: ShortRoleNameFilter, $page: Page, $sort: Sort) {
      servers(filter: {id: $serverId}) {
        shortRoleNameCount
        shortRoleNames(filter: $filter, page: $page, sort: $sort) {
          id
          shortName
          roleId
          roleName
          position
          server {
            id
            name
          }
          createdAt
          modifiedAt
        }
      }
    }
  `;

  static shortRoleNameWithHistoryQuery = `
    query ShortRoleNameListHistory($serverId: ID, $id: ID) {
      servers(filter: {id: $serverId}) {
        shortRoleNameCount
        shortRoleNames(filter: {id: $id}) {
          id

          history {
            id
            shortName
            roleId
            roleName
            position
            server
            deleted
            dateFrom
            dateTo
          }
        }
      }
    }
  `;

  static usersQuery = `
    query UsersList($serverId: ID, $filter: UserFilter, $page: Page, $sort: Sort) {
      servers(filter: {id: $serverId}) {
        userCount
        users(filter: $filter, page: $page, sort: $sort) {
          id
          discordId
          name
          xp
          ontime
          level {
            id
            name
          }
          leftServer

          createdAt
          modifiedAt
        }
      }
    }
  `;

  static userProfile = `
    query UserProfile($serverId: ID, $userId: ID, $page: Page, $sort: Sort) {
      userCount
      users(filter: {server: {id: $serverId}, id: $userId}, page: $page, sort: $sort) {
        id
        discordId
        name
        xp
        messageCount
        reactionCount
        birthday
        ontime
        level {
          id
          name
        }
        leftServer
        server {
          id
          name
        }

        joinedServerCount
        joinedServers {
          id
          joinedOn
          leavedOn
        }

        createdAt
        modifiedAt
      }
    }
  `;

  static userProfileAchievements = `
    query UserProfile($serverId: ID, $userId: ID, $page: Page, $sort: Sort) {
      users(filter: {server: {id: $serverId}, id: $userId}, page: $page, sort: $sort) {
        achievementCount
        achievements {
          id
          name
          description
          createdAt
        }
      }
    }
  `;

  static userProfileVoiceChannelJoins = `
    query UserProfile($serverId: ID, $userId: ID, $page: Page, $sort: Sort) {
      users(filter: {server: {id: $serverId}, id: $userId}, page: $page, sort: $sort) {
        joinedVoiceChannelCount
        joinedVoiceChannels {
          id
          channelId
          channelName
          time
          joinedOn
          leavedOn
        }
      }
    }
  `;

  static userProfileGameserverJoins = `
    query UserProfile($serverId: ID, $userId: ID, $page: Page, $sort: Sort) {
      users(filter: {server: {id: $serverId}, id: $userId}, page: $page, sort: $sort) {
        userJoinedGameServerCount
        userJoinedGameServers {
          id
          gameServer
          time
          joinedOn
          leavedOn
        }
      }
    }
  `;

  static userProfileWarnings = `
    query UserProfile($serverId: ID, $userId: ID, $page: Page, $sort: Sort) {
      users(filter: {server: {id: $serverId}, id: $userId}, page: $page, sort: $sort) {
        userWarningCount
        userWarnings {
          id
          user {
            id
            name
          }
          description
          author {
            id
            name
          }

          createdAt
        }
      }
    }
  `;

  static userQueryWithHistory = `
    query UsersWithHistory($serverId: ID, $id: ID) {
      servers(filter: {id: $serverId}) {
        userCount
        users(filter: {id: $id}) {
          id

          history {
            id
            discordId
            xp
            server
            deleted
            dateFrom
            dateTo
          }
        }
      }
    }
  `;

  static autoRolesQuery = `
    query AutoRoleQuery($serverId: ID, $filter: AutoRoleFilter, $page: Page, $sort: Sort) {
      servers(filter: {id: $serverId}) {
        autoRoleCount
        autoRoles(filter: $filter, page: $page, sort: $sort) {
          id
          channelId
          channelName
          messageId
          autoRoleRuleCount

          server {
            id
          }

          createdAt
          modifiedAt
        }
      }
    }
  `;

  static autoRolesWithHistoryQuery = `
    query AutoRoleWithHistoryQuery($serverId: ID, $id: ID) {
      servers(filter: {id: $serverId}) {
        autoRoleCount
        autoRoles(filter: {id: $id}) {
          id

          history {
            id
            channelId
            messageId
            server
            deleted
            dateFrom
            dateTo
          }
        }
      }
    }
  `;

  static autoRoleRulesQuery = `
    query AutoRoleRuleQuery($serverId: ID, $autoRoleId: ID, $filter: AutoRoleRuleFilter, $page: Page, $sort: Sort) {
      servers(filter: {id: $serverId}) {
        autoRoles(filter: {id: $autoRoleId}) {
          autoRoleRuleCount
          autoRoleRules(filter: $filter, page: $page, sort: $sort) {
            id
            emojiName
            roleId
            roleName

            autoRole {
              id
            }

            createdAt
            modifiedAt
          }
        }
      }
    }
  `;

  static autoRoleRulesHistoryQuery = `
    query AutoRoleRuleHistoryQuery($serverId: ID, $autoRoleId: ID, $id: ID) {
      servers(filter: {id: $serverId}) {
        autoRoles(filter: {id: $autoRoleId}) {
          autoRoleRuleCount
          autoRoleRules(filter: {id: $id}) {
            id

            history {
              id
              emojiName
              roleId
              autoRole
              deleted
              dateFrom
              dateTo
            }
          }
        }
      }
    }
  `;

  static technicianConfigQuery = `
    query technicianConfigQuery {
      technicianConfig {
        id
        helpCommandReferenceUrl
        waitForRestart
        waitForShutdown
        cacheMaxMessages
        featureFlags {
          key
          value
        }
        pingURLs
        technicianIds

        createdAt
        modifiedAt
      }
    }
  `;

  static serverConfigQuery = `
    query serverConfigQuery($serverId: ID) {
      servers(filter: { id: $serverId }) {
        name
        config {
          id
          messageDeleteTimer
          notificationChatId
          maxVoiceStateHours
          xpPerMessage
          xpPerReaction
          maxMessageXpPerHour
          xpPerOntimeHour
          xpPerEventParticipation
          xpPerAchievement
          xpForBirthday
          afkCommandChannelId
          helpVoiceChannelId
          teamChannelId
          loginMessageChannelId
          defaultRoleId
          shortRoleNameOnlySetHighestRole
          featureFlags {
            key
            value
          }
          afkChannelIds
          moderatorRoleIds
          adminRoleIds

          server {
            id
          }
        }
      }
    }
  `;
}
