import { Component, EventEmitter, Input, OnInit, Output } from "@angular/core";
import { Table } from "primeng/table";

@Component({
  selector: "app-config-list",
  templateUrl: "./config-list.component.html",
  styleUrls: ["./config-list.component.scss"]
})
export class ConfigListComponent {
  internal_data: any[] = [];

  @Input() translationKey: string = "";
  @Input() options?: Array<any>;
  @Input() optionLabel?: string = "";
  @Input() optionValue?: string = "";

  @Input()
  set data(val: any[]) {
    this.dataChange.emit(val);
    let id = 0;
    this.internal_data = val.map(value => {
      value = { id: id, value: value };
      id++;
      return value;
    });
  }

  get data() {
    return this.getData();
  }

  @Output() dataChange: EventEmitter<any> = new EventEmitter<any>();
  clonedData: { [s: number]: any } = {};

  private getData(): any[] {
    return this.internal_data.map(value => {
      return value.value;
    });
  }

  addNew(table: Table) {
    const id = this.internal_data.length == 0 ? 1 : Math.max.apply(Math, this.internal_data.map(value => {
      return value.id ?? 0;
    })) + 1;
    const newItem = { id: id, value: "" };
    this.internal_data.push(newItem);

    table.initRowEdit(newItem);
    const index = this.internal_data.findIndex(l => l.id == newItem.id);
    this.editInit(newItem, index);
  }

  editInit(value: any, index: number) {
    this.clonedData[index] = { ...value };
  }

  delete(index: number) {
    this.internal_data.splice(index, 1);
    this.dataChange.emit(this.getData());
  }

  editSave(value: any, index: number) {
    if (!value.value || this.internal_data[index] == this.clonedData[index]) {
      return;
    }

    delete this.clonedData[index];
    this.dataChange.emit(this.getData());
  }

  editCancel(index: number) {
    this.internal_data[index] = this.clonedData[index];
    delete this.clonedData[index];
  }
}
